#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/11/14
 """
from __future__ import unicode_literals, absolute_import


PAY_RESULT_NOT_FOUND = 6000
REFUND_AMOUNT_NOT_RIGHT = 6001